@@ -32,12 +32,12 @@ DEFAULT_MEDIA_TYPE = 'application/json; charset=utf-8'
 
 
 # Hoist classes and functions into the falcon namespace
-from .version import version  # NOQA
-from .api import API, DEFAULT_MEDIA_TYPE  # NOQA
-from .status_codes import *  # NOQA
-from .exceptions import *  # NOQA
-from .http_error import HTTPError  # NOQA
-from .util import dt_to_http  # NOQA
-from .hooks import before, after  # NOQA
-from .request import Request  # NOQA
-from .response import Response  # NOQA
\ No newline at end of file
+from falcon.version import version  # NOQA
+from falcon.api import API, DEFAULT_MEDIA_TYPE  # NOQA
+from falcon.status_codes import *  # NOQA
+from falcon.exceptions import *  # NOQA
+from falcon.http_error import HTTPError  # NOQA
+from falcon.util import dt_to_http  # NOQA
+from falcon.hooks import before, after  # NOQA
+from falcon.request import Request  # NOQA
+from falcon.response import Response  # NOQA
